from rest_framework import serializers

class UserSerializer(serializers.Serializer):
    id = serializers.CharField(max_length=255)
    email = serializers.EmailField()
    name = serializers.CharField(max_length=255)
    emailVerifiedAt = serializers.DateTimeField()
    isTwoFactor = serializers.BooleanField()
    authProvider = serializers.CharField(max_length=255)

class RegisterUserSerializer(serializers.Serializer):
    email = serializers.EmailField()
    password = serializers.CharField(max_length=255)
    name = serializers.CharField(max_length=255)
    isTwoFactor = serializers.BooleanField()
    emailVerifiedAt = serializers.DateTimeField()

class LoginUserSerializer(serializers.Serializer):
    email = serializers.EmailField()
    password = serializers.CharField(max_length=255)
    authProvider = serializers.CharField(max_length=255)

class ResetPasswordSerializer(serializers.Serializer):
    email = serializers.EmailField()
    key = serializers.CharField(max_length=255)
    password = serializers.CharField(max_length=255)

class UpdatePasswordSerializer(serializers.Serializer):
    oldPassword = serializers.CharField(max_length=255)
    newPassword = serializers.CharField(max_length=255)

class VerificationKeySerializer(serializers.Serializer):
    email = serializers.EmailField()
    callBackUrl = serializers.URLField()

class VerifyKeySerializer(serializers.Serializer):
    email = serializers.EmailField()
    key = serializers.CharField(max_length=255)

class EmailSerializer(serializers.Serializer):
    email = serializers.EmailField()

class PasswordResetSerializer(serializers.Serializer):
    email = serializers.EmailField()
    callBackUrl = serializers.URLField()

class SessionSerializer(serializers.Serializer):
    sessionToken = serializers.CharField(max_length=255)
    refreshToken = serializers.CharField(max_length=255)

class OAuthLoginSerializer(serializers.Serializer):
    originUrl = serializers.URLField()

class UpsertOAuthUserSerializer(serializers.Serializer):
    provider = serializers.CharField(max_length=255)
    providerUserId = serializers.CharField(max_length=255)
    accessToken = serializers.CharField(max_length=255)
    refreshToken = serializers.CharField(max_length=255)
    name = serializers.CharField(max_length=255)
    pictureUrl = serializers.URLField()
    email = serializers.EmailField()



from django.urls import path, re_path
from drf_yasg import openapi
from drf_yasg.views import get_schema_view
from rest_framework import permissions

from .views import (
    RegisterUserView, LoginUserView, ResetPasswordView, UpdatePasswordView,
    SendVerificationKeyView, VerifyKeyView, FindExistingUserView, SendPasswordResetView,
    VerifyPasswordResetView, ValidateSessionView, RefreshSessionView, LogoutUserView,
    GoogleOAuthLoginView, AppleOAuthLoginView, UpsertOAuthUserView
)

schema_view = get_schema_view(
    openapi.Info(
        title="API Documentation",
        default_version='v1',
        description="API documentation for credential and session management.",
        terms_of_service="https://www.google.com/policies/terms/",
        contact=openapi.Contact(email="contact@runners.im"),
        license=openapi.License(name="BSD License"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)

urlpatterns = [
    path('swagger/', schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
    path('redoc/', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),
    re_path(r'^swagger(?P<format>\.json|\.yaml)$', schema_view.without_ui(cache_timeout=0), name='schema-json'),
    path('credential/register', RegisterUserView.as_view(), name='register-user'),
    path('credential/login', LoginUserView.as_view(), name='login-user'),
    path('credential/reset-password', ResetPasswordView.as_view(), name='reset-password'),
    path('credential/update-password', UpdatePasswordView.as_view(), name='update-password'),
    path('validate/send-verification-key', SendVerificationKeyView.as_view(), name='send-verification-key'),
    path('validate/verify-key', VerifyKeyView.as_view(), name='verify-key'),
    path('validate/find-existing-user', FindExistingUserView.as_view(), name='find-existing-user'),
    path('validate/send-password-reset', SendPasswordResetView.as_view(), name='send-password-reset'),
    path('validate/verify-password-reset', VerifyPasswordResetView.as_view(), name='verify-password-reset'),
    path('session/validate', ValidateSessionView.as_view(), name='validate-session'),
    path('session/refresh', RefreshSessionView.as_view(), name='refresh-session'),
    path('session/logout', LogoutUserView.as_view(), name='logout-user'),
    path('oauth/google', GoogleOAuthLoginView.as_view(), name='google-oauth-login'),
    path('oauth/apple', AppleOAuthLoginView.as_view(), name='apple-oauth-login'),
    path('oauth/upsertUser', UpsertOAuthUserView.as_view(), name='upsert-oauth-user'),
]



import requests
from django.conf import settings
from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from .serializers import (
    RegisterUserSerializer, LoginUserSerializer, ResetPasswordSerializer,
    UpdatePasswordSerializer, VerificationKeySerializer, VerifyKeySerializer,
    EmailSerializer, PasswordResetSerializer, SessionSerializer, OAuthLoginSerializer,
    UpsertOAuthUserSerializer
)

BASE_URL = settings.BASE_URL


class RegisterUserView(APIView):
    @swagger_auto_schema(
        request_body=RegisterUserSerializer,
        responses={
            200: openapi.Response('Existing user', RegisterUserSerializer),
            201: openapi.Response('User created', RegisterUserSerializer),
            400: 'Invalid request',
            500: 'Server error'
        }
    )
    def post(self, request):
        serializer = RegisterUserSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/credential/register", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class LoginUserView(APIView):
    @swagger_auto_schema(
        request_body=LoginUserSerializer,
        responses={
            200: openapi.Response('Login successful', LoginUserSerializer),
            400: 'Bad request',
            500: 'Internal server error'
        }
    )
    def post(self, request):
        serializer = LoginUserSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/credential/login", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class ResetPasswordView(APIView):
    @swagger_auto_schema(
        request_body=ResetPasswordSerializer,
        responses={
            200: openapi.Response('Password reset successfully'),
            400: 'Invalid request',
            500: 'Internal server error'
        }
    )
    def post(self, request):
        serializer = ResetPasswordSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/credential/reset-password", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class UpdatePasswordView(APIView):
    @swagger_auto_schema(
        request_body=UpdatePasswordSerializer,
        responses={
            200: openapi.Response('Password updated successfully'),
            400: 'Invalid request',
            500: 'Internal server error'
        },
        manual_parameters=[openapi.Parameter('Authorization', openapi.IN_HEADER, description="Bearer sessionToken",
                                             type=openapi.TYPE_STRING)]
    )
    def post(self, request):
        serializer = UpdatePasswordSerializer(data=request.data)
        if serializer.is_valid():
            headers = {'Authorization': f"Bearer {request.headers.get('Authorization')}"}
            response = requests.post(f"{BASE_URL}/credential/update-password", json=serializer.validated_data,
                                     headers=headers)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class SendVerificationKeyView(APIView):
    @swagger_auto_schema(
        request_body=VerificationKeySerializer,
        responses={
            200: openapi.Response('Email verification sent'),
            400: 'Invalid request',
            500: 'Internal server error'
        }
    )
    def post(self, request):
        serializer = VerificationKeySerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/validate/send-verification-key", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class VerifyKeyView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('email', openapi.IN_QUERY, description="Email", type=openapi.TYPE_STRING),
            openapi.Parameter('key', openapi.IN_QUERY, description="Verification key", type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('Email verified'),
            400: 'Invalid request',
            401: 'Unauthorized',
            500: 'Internal server error'
        }
    )
    def get(self, request):
        serializer = VerifyKeySerializer(data=request.query_params)
        if serializer.is_valid():
            response = requests.get(f"{BASE_URL}/validate/verify-key", params=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class FindExistingUserView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('email', openapi.IN_QUERY, description="Email", type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('User found', EmailSerializer),
            400: 'Invalid request'
        }
    )
    def get(self, request):
        serializer = EmailSerializer(data=request.query_params)
        if serializer.is_valid():
            response = requests.get(f"{BASE_URL}/validate/find-existing-user", params=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class SendPasswordResetView(APIView):
    @swagger_auto_schema(
        request_body=PasswordResetSerializer,
        responses={
            200: openapi.Response('Password reset sent'),
            400: 'Invalid request',
            500: 'Internal server error'
        }
    )
    def post(self, request):
        serializer = PasswordResetSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/validate/send-password-reset", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class VerifyPasswordResetView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('email', openapi.IN_QUERY, description="Email", type=openapi.TYPE_STRING),
            openapi.Parameter('key', openapi.IN_QUERY, description="Reset key", type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('Password reset verified'),
            400: 'Invalid request',
            500: 'Internal server error'
        }
    )
    def get(self, request):
        serializer = VerifyKeySerializer(data=request.query_params)
        if serializer.is_valid():
            response = requests.get(f"{BASE_URL}/validate/verify-password-reset", params=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class ValidateSessionView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('Authorization', openapi.IN_HEADER, description="Bearer sessionToken",
                              type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('Session valid'),
            401: 'Invalid session',
            500: 'Internal server error'
        }
    )
    def get(self, request):
        headers = {'Authorization': f"Bearer {request.headers.get('Authorization')}"}
        response = requests.get(f"{BASE_URL}/session/validate", headers=headers)
        return Response(response.json(), status=response.status_code)


class RefreshSessionView(APIView):
    @swagger_auto_schema(
        request_body=SessionSerializer,
        responses={
            200: openapi.Response('Session refreshed', SessionSerializer),
            400: 'Invalid request',
            401: 'Invalid refresh token',
            500: 'Internal server error'
        }
    )
    def post(self, request):
        serializer = SessionSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/session/refresh", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class LogoutUserView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('Authorization', openapi.IN_HEADER, description="Bearer sessionToken",
                              type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('User logged out'),
            400: 'Invalid request'
        }
    )
    def post(self, request):
        headers = {'Authorization': f"Bearer {request.headers.get('Authorization')}"}
        response = requests.post(f"{BASE_URL}/session/logout", headers=headers)
        return Response(response.json(), status=response.status_code)


class GoogleOAuthLoginView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('originUrl', openapi.IN_QUERY, description="The URL to redirect to after login",
                              type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('Login successful'),
            400: 'Invalid Google ID token',
            500: 'Internal server error'
        }
    )
    def get(self, request):
        serializer = OAuthLoginSerializer(data=request.query_params)
        if serializer.is_valid():
            response = requests.get(f"{BASE_URL}/oauth/google", params=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class AppleOAuthLoginView(APIView):
    @swagger_auto_schema(
        manual_parameters=[
            openapi.Parameter('originUrl', openapi.IN_QUERY, description="The URL to redirect to after login",
                              type=openapi.TYPE_STRING)
        ],
        responses={
            200: openapi.Response('Login successful'),
            400: 'Invalid token',
            500: 'Internal server error'
        }
    )
    def get(self, request):
        serializer = OAuthLoginSerializer(data=request.query_params)
        if serializer.is_valid():
            response = requests.get(f"{BASE_URL}/oauth/apple", params=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class UpsertOAuthUserView(APIView):
    @swagger_auto_schema(
        request_body=UpsertOAuthUserSerializer,
        responses={
            200: openapi.Response('Login successful', UpsertOAuthUserSerializer),
            400: 'Invalid OAuth provider',
            500: 'Failed to update OAuth account'
        }
    )
    def post(self, request):
        serializer = UpsertOAuthUserSerializer(data=request.data)
        if serializer.is_valid():
            response = requests.post(f"{BASE_URL}/oauth/upsertUser", json=serializer.validated_data)
            return Response(response.json(), status=response.status_code)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



import os
import requests
from django.conf import settings

BASE_URL = settings.BASE_URL

def register_user(email, password, name, is_two_factor, email_verified_at):
    url = f"{BASE_URL}/credential/register"
    payload = {
        "email": email,
        "password": password,
        "name": name,
        "isTwoFactor": is_two_factor,
        "emailVerifiedAt": email_verified_at
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def login_user(email, password):
    url = f"{BASE_URL}/credential/login"
    payload = {
        "email": email,
        "password": password,
        "authProvider": "credential"
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def reset_password(email, key, password):
    url = f"{BASE_URL}/credential/reset-password"
    payload = {
        "email": email,
        "key": key,
        "password": password
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def update_password(session_token, old_password, new_password):
    url = f"{BASE_URL}/credential/update-password"
    payload = {
        "oldPassword": old_password,
        "newPassword": new_password
    }
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {session_token}'
    }
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def send_verification_key(email, call_back_url):
    url = f"{BASE_URL}/validate/send-verification-key"
    payload = {
        "email": email,
        "callBackUrl": call_back_url
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def verify_key(email, key):
    url = f"{BASE_URL}/validate/verify-key"
    params = {
        "email": email,
        "key": key
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.get(url, params=params, headers=headers)
    return response.json(), response.status_code

def find_existing_user(email):
    url = f"{BASE_URL}/validate/find-existing-user"
    params = {"email": email}
    headers = {'Content-Type': 'application/json'}
    response = requests.get(url, params=params, headers=headers)
    return response.json(), response.status_code

def send_password_reset(email, call_back_url):
    url = f"{BASE_URL}/validate/send-password-reset"
    payload = {
        "email": email,
        "callBackUrl": call_back_url
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def verify_password_reset(email, key):
    url = f"{BASE_URL}/validate/verify-password-reset"
    params = {
        "email": email,
        "key": key
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.get(url, params=params, headers=headers)
    return response.json(), response.status_code

def validate_session(session_token):
    url = f"{BASE_URL}/session/validate"
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {session_token}'
    }
    response = requests.get(url, headers=headers)
    return response.json(), response.status_code

def refresh_session(refresh_token):
    url = f"{BASE_URL}/session/refresh"
    payload = {"refreshToken": refresh_token}
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code

def logout_user(session_token):
    url = f"{BASE_URL}/session/logout"
    headers = {
        'Content-Type': 'application/json',
        'Authorization': f'Bearer {session_token}'
    }
    response = requests.post(url, headers=headers)
    return response.json(), response.status_code

def google_oauth_login(origin_url):
    url = f"{BASE_URL}/oauth/google"
    params = {"originUrl": origin_url}
    headers = {'Content-Type': 'application/json'}
    response = requests.get(url, params=params, headers=headers)
    return response.json(), response.status_code

def apple_oauth_login(origin_url):
    url = f"{BASE_URL}/oauth/apple"
    params = {"originUrl": origin_url}
    headers = {'Content-Type': 'application/json'}
    response = requests.get(url, params=params, headers=headers)
    return response.json(), response.status_code

def upsert_oauth_user(provider, provider_user_id, access_token, refresh_token, name, picture_url, email):
    url = f"{BASE_URL}/oauth/upsertUser"
    payload = {
        "provider": provider,
        "providerUserId": provider_user_id,
        "accessToken": access_token,
        "refreshToken": refresh_token,
        "name": name,
        "pictureUrl": picture_url,
        "email": email
    }
    headers = {'Content-Type': 'application/json'}
    response = requests.post(url, json=payload, headers=headers)
    return response.json(), response.status_code



import hashlib
from django.test import TestCase
from .gateway import login_user, validate_session
from django.conf import settings

def hash_password(password):
    sha_signature = hashlib.sha256(password.encode()).hexdigest()
    return sha_signature

class GatewayTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        settings.BASE_URL = 'https://cauth-dev.creta.world'

    def test_login_and_validate(self):
        # User credentials
        email = "admin@runners.im"
        password = "run0601!"
        hashed_password = hash_password(password)
        print('hashed_password: ', hashed_password)

        # Login the user
        login_response, login_status = login_user(email, hashed_password)
        print('login_response: ', login_response)

        # Check if login was successful
        self.assertEqual(login_status, 200, f"Login failed: {login_response}")

        session_token = login_response.get('session', {}).get('sessionToken')
        print('session_token: ', session_token)
        self.assertIsNotNone(session_token, "Login did not return a session token")

        # Validate the session
        validate_response, validate_status = validate_session(session_token)
        print('validate_response: ', validate_response)

        # Check if session validation was successful
        self.assertEqual(validate_status, 200, f"Session validation failed: {validate_response}")
        self.assertTrue(validate_response.get('isValid'), "Session is not valid")
